import pytest

from ceo_common.errors import GroupNotFoundError


def test_group_add_to_ldap(simple_group, ldap_srv):
    group = simple_group

    group.add_to_ldap()
    retrieved_group = ldap_srv.get_group(group.cn)
    assert retrieved_group.cn == group.cn

    group.remove_from_ldap()
    with pytest.raises(GroupNotFoundError):
        ldap_srv.get_group(group.cn)


def test_group_members(ldap_group, ldap_srv):
    group = ldap_group

    group.add_member('member1')
    assert group.members == ['member1']
    assert ldap_srv.get_group(group.cn).members == group.members

    group.add_member('member2')
    assert group.members == ['member1', 'member2']
    assert ldap_srv.get_group(group.cn).members == group.members

    group.remove_member('member1')
    assert group.members == ['member2']
    assert ldap_srv.get_group(group.cn).members == group.members


def test_group_to_dict(simple_group):
    group = simple_group

    expected = {
        'cn': group.cn,
        'gid_number': group.gid_number,
        'members': group.members,
    }
    assert group.to_dict() == expected
